---- ##########################################################################################################
---- #                                                                                                        #
---- # GPS INFO                   			                                                                      #
-----#                                                                                                        #
---- # License GPLv3: http://www.gnu.org/licenses/gpl-3.0.html                                                #
---- #                                                                                                        #
---- # This program is free software; you can redistribute it and/or modify                                   #
---- # it under the terms of the GNU General Public License version 3 as                                      #
---- # published by the Free Software Foundation.                                                             #
---- #                                                                                                        #
---- #                                                                                                        #
---- # Björn Pasteuning / Hobby4life 2020                                                                     #
---- # Website: https://www.hobby4life.nl                                                                     #
---- #                                                                                                        #
---- ##########################################################################################################




local options = {
  { "ResetAll"  , SOURCE  , 1     },  --Define a source for trigger a reset of all  values
  { "Imperial"  , VALUE   , 0,0,1 },  --Toggle between Metric or Imperial notation, note that correct settings have to be set on the sensor page too!
}


-- in the create function you add all shared variables to the array containing the widget data ('thisWidget')
local function create(zone, options)
  
  local thisWidget  = {zone=zone, options=options,counter = 0}

-- Declaration and preset of Global Widget Variables used in the entire scope --
  LoSDistance = 0
  
  --create array containing all sensor ID's used for quicker retrieval
  local ID = {}
  ID.GPS        = getFieldInfo("GPS")  and getFieldInfo("GPS").id	 or 0
  ID.GSpd       = getFieldInfo("GSpd") and getFieldInfo("GSpd").id or 0
  ID.GAlt       = getFieldInfo("GAlt") and getFieldInfo("GAlt").id or 0  -- Vario Altimeter has priority over GPS Altimeter
  ID.Tmp1       = getFieldInfo("Tmp1") and getFieldInfo("Tmp1").id or 0 -- used with OpenXsenor or sallites in view indicator
  ID.Tmp2       = getFieldInfo("Tmp2") and getFieldInfo("Tmp2").id or 0  
  ID.Hdg        = getFieldInfo("Hdg")  and getFieldInfo("Hdg").id  or 0
  
  --add ID to thisWidget
  thisWidget.ID = ID	
 
  --return the thisWidget array to the opentx API, containing all data to be shared across functions
  return thisWidget
  
end

local function update(thisWidget, options)
  thisWidget.options = options
end

--***********************************************************************
--*                        BACKGROUND FUNCTION                          *
--***********************************************************************
local function background(thisWidget)
  
  ImperialSet = thisWidget.options.Imperial or 0
  
  thisWidget.gpsLatLong = getValue(thisWidget.ID.GPS)
  
  
  
  if  (type(thisWidget.gpsLatLong) ~= "table") then
    thisWidget.ID.GPS       = getFieldInfo("GPS")  and getFieldInfo("GPS").id	 or 0
    thisWidget.ID.GSpd      = getFieldInfo("GSpd") and getFieldInfo("GSpd").id or 0
    thisWidget.ID.GAlt      = getFieldInfo("GAlt") and getFieldInfo("GAlt").id or 0
    thisWidget.ID.Tmp1      = getFieldInfo("Tmp1") and getFieldInfo("Tmp1").id or 0
    thisWidget.ID.Tmp2      = getFieldInfo("Tmp2") and getFieldInfo("Tmp2").id or 0
    thisWidget.ID.Hdg       = getFieldInfo("Hdg")  and getFieldInfo("Hdg").id  or 0
    return
  end
  
  thisWidget.Plane_Speed      = getValue(thisWidget.ID.GSpd)
  thisWidget.Plane_Altitude   = getValue(thisWidget.ID.GAlt)
  thisWidget.Plane_Tmp1       = getValue(thisWidget.ID.Tmp1)
  thisWidget.Plane_Tmp2       = getValue(thisWidget.ID.Tmp2)
  thisWidget.Plane_Hdg        = getValue(thisWidget.ID.Hdg)

  
  thisWidget.Plane_gpsLat     = thisWidget.gpsLatLong.lat
  thisWidget.Plane_gpsLong    = thisWidget.gpsLatLong.lon
  

end
---------------------------------------------------------------------------------------------------------

--***********************************************************************
--*                          SPECIAL FUNCTIONS                          *
--***********************************************************************

----------------------- Function to calculated bearing angle between 2 coordinates ----------------------
function CalcBearing(PrevLat,PrevLong,NewLat,NewLong)
  local yCalc = math.sin(math.rad(NewLong)-math.rad(PrevLong)) * math.cos(math.rad(NewLat))
  local xCalc = math.cos(math.rad(PrevLat)) * math.sin(math.rad(NewLat)) - math.sin(math.rad(PrevLat)) * math.cos(math.rad(NewLat)) * math.cos(math.rad(NewLat) - math.rad(PrevLat))
  local Bearing = math.deg(math.atan2(yCalc,xCalc))
  if Bearing < 0 then
    Bearing = 360 + Bearing
  end  
  return Bearing
end

----------------------- Function to calculate distance between 2 coordinates -----------------------------
function CalcDistance(PrevLat,PrevLong,NewLat,NewLong)
  local earthRadius = 0
  if ImperialSet == 1 then
    earthRadius = 20902000  --feet  --3958.8 miles
  else
    earthRadius = 6371000   --meters
  end
  local dLat = math.rad(NewLat-PrevLat)
  local dLon = math.rad(NewLong-PrevLong)
  PrevLat = math.rad(PrevLat)
  NewLat = math.rad(NewLat)
  local a = math.sin(dLat/2) * math.sin(dLat/2) + math.sin(dLon/2) * math.sin(dLon/2) * math.cos(PrevLat) * math.cos(NewLat) 
  local c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
  return (earthRadius * c)
end    

--***********************************************************************
--*                            UPDATE FUNCTION                          *
--***********************************************************************
local function update(thisWidget, options)
  thisWidget.options = options
end

--***********************************************************************
--*                           REFRESH FUNCTION                          *
--***********************************************************************
local function refresh(thisWidget)
  
  background(thisWidget)
  
  Pilot_GPSTable = getTxGPS()
  
  local Pilot_NS  = ""
  local Pilot_EW  = ""
  local Plane_NS  = ""
  local Plane_EW  = ""  
  local FM        = ""
  local SPD       = ""
  local Plane_Lat   = thisWidget.Plane_gpsLat or 0
  local Plane_Long  = thisWidget.Plane_gpsLong or 0 
  local Plane_Speed = thisWidget.Plane_Speed or 0
  local Plane_Alt   = thisWidget.Plane_Altitude or 0
  local Plane_Sats  = thisWidget.Plane_Tmp1 or 0  
  local Plane_Hdop  = thisWidget.Plane_Tmp2 or 0
  local Plane_Hdg   = thisWidget.Plane_Hdg or 0
  local Plane_Fix   = false
  
  local Pilot_Lat   = Pilot_GPSTable.lat 
  local Pilot_Long  = Pilot_GPSTable.lon 
  local Pilot_Alt   = Pilot_GPSTable.alt 
  local Pilot_Hdg   = Pilot_GPSTable.heading
  local Pilot_Speed = Pilot_GPSTable.speed
  local Pilot_Fix   = Pilot_GPSTable.fix 
  local Pilot_Sats  = Pilot_GPSTable.numsat  
  
 
  local PilotToPlaneDistance = 0
  
  if Plane_Sats > 100 then
    Plane_Sats = math.abs(Plane_Sats  - 100)
    Plane_Fix  = true
  else
    Plane_Sats = 0
    Plane_Fix  = false
  end
  

  
  
  if (Pilot_Fix==true) then
    PilotToPlaneDistance = CalcDistance(Pilot_Lat,Pilot_Long,Plane_Lat,Plane_Long) or 0
  else
    PilotToPlaneDistance = 0
  end  
  
------------------------------ Checks if valid GPS data is received -------------------------------------  
 if  (type(thisWidget.gpsLatLong) ~= "table") then
	lcd.setColor(CUSTOM_COLOR, lcd.RGB(255,0,0))
  lcd.drawFilledRectangle(0, 239, 480, 33, SOLID)    
	lcd.drawText( 120, 236, "No GPS SIGNAL", DBLSIZE + CUSTOM_COLOR + SHADOWED)
	return
end

  ----------- Corrects some readout internal X12S GPS ----------------------
  Pilot_Hdg = Pilot_Hdg / 10
  Pilot_Speed = Pilot_Speed / 10

------------ Calculates Heading and Bearing from previous and new location of the plane ----------------------

  Bearing = CalcBearing(Pilot_Lat,Pilot_Long,Plane_Lat,Plane_Long)
  headingDeg = Plane_Hdg or 0

-- Preset info
	if Plane_Lat > 0 then
		Plane_NS = "N" 
	else
		Plane_NS = "S"
	end

	if Plane_Long > 0 then
		Plane_EW = "E"
	else
		Plane_EW = "W" 
	end
  
	if Pilot_Lat > 0 then
		Pilot_NS = "N" 
	else
		Pilot_NS = "S"
	end

	if Pilot_Long > 0 then
		Pilot_EW = "E"
	else
		Pilot_EW = "W" 
	end  
	
	if ImperialSet == 1 then
		FM  = "ft"
    SPD = "mph"
	else
		FM  = "m"
    SPD = "kmh"
	end  
  
  if thisWidget.zone.w == 460 then
  
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawRectangle(thisWidget.zone.x -5, thisWidget.zone.y -5, thisWidget.zone.w + 10, thisWidget.zone.h + 10, SOLID + CUSTOM_COLOR) 
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawRectangle(thisWidget.zone.x -4, thisWidget.zone.y -4, thisWidget.zone.w + 8, thisWidget.zone.h + 8, SOLID + CUSTOM_COLOR)     
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawRectangle(thisWidget.zone.x -3, thisWidget.zone.y -3, thisWidget.zone.w + 6, thisWidget.zone.h + 6, SOLID + CUSTOM_COLOR)     
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))        
    lcd.drawText(thisWidget.zone.x , thisWidget.zone.y - 5, "==X12S GPS INFORMATION ==", CUSTOM_COLOR + DBLSIZE +SHADOWED)
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 30, thisWidget.zone.w + 12, thisWidget.zone.y + 30, SOLID , CUSTOM_COLOR)
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 32, thisWidget.zone.w + 12, thisWidget.zone.y + 32, SOLID , CUSTOM_COLOR)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawLine(thisWidget.zone.x -4, thisWidget.zone.y + 31, thisWidget.zone.w + 13, thisWidget.zone.y + 31, SOLID , CUSTOM_COLOR)
    
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x, thisWidget.zone.y + 32, "Sats: "..Pilot_Sats, LEFT + CUSTOM_COLOR + SHADOWED);
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,0))
    lcd.drawText(thisWidget.zone.x + 95, thisWidget.zone.y + 32, "PILOT", CUSTOM_COLOR + SHADOWED)
   
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawLine((LCD_W / 2), thisWidget.zone.y + 32, (LCD_W / 2), thisWidget.zone.y + 50, SOLID , CUSTOM_COLOR)
    lcd.drawLine((LCD_W / 2)  + 2, thisWidget.zone.y + 32, (LCD_W / 2) + 2, thisWidget.zone.y + 50, SOLID , CUSTOM_COLOR)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawLine((LCD_W / 2) + 1, thisWidget.zone.y + 31, (LCD_W / 2) + 1, thisWidget.zone.y + 51, SOLID , CUSTOM_COLOR)   
   
   
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x + 235, thisWidget.zone.y + 32, "Sats: "..Plane_Sats, LEFT + CUSTOM_COLOR + SHADOWED);
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,0))
    lcd.drawText(thisWidget.zone.x + 320, thisWidget.zone.y + 32, "PLANE", CUSTOM_COLOR + SHADOWED)
    
    
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 50, thisWidget.zone.w + 12, thisWidget.zone.y + 50, SOLID , CUSTOM_COLOR)
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 52, thisWidget.zone.w + 12, thisWidget.zone.y + 52, SOLID , CUSTOM_COLOR)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawLine(thisWidget.zone.x -4, thisWidget.zone.y + 51, thisWidget.zone.w + 13, thisWidget.zone.y + 51, SOLID , CUSTOM_COLOR)
    
        
    ------------------- PILOT COORDINATES-----------------------
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x , thisWidget.zone.y + 52, "Lat:", CUSTOM_COLOR + MIDSIZE +SHADOWED)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,252,0))
    lcd.drawText(thisWidget.zone.x + 55, thisWidget.zone.y + 52, Pilot_NS.." "..Pilot_Lat, CUSTOM_COLOR + MIDSIZE +SHADOWED)    

    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x , thisWidget.zone.y + 77, "Lon:", CUSTOM_COLOR + MIDSIZE +SHADOWED)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,0))
    lcd.drawText(thisWidget.zone.x + 55, thisWidget.zone.y + 77, Pilot_EW.." " ..Pilot_Long, CUSTOM_COLOR + MIDSIZE +SHADOWED)    
    
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawLine((LCD_W / 2), thisWidget.zone.y + 52, (LCD_W / 2), thisWidget.zone.y + 107, SOLID , CUSTOM_COLOR)
    lcd.drawLine((LCD_W / 2)  + 2, thisWidget.zone.y + 52, (LCD_W / 2) + 2, thisWidget.zone.y + 107, SOLID , CUSTOM_COLOR)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawLine((LCD_W / 2) + 1, thisWidget.zone.y + 51, (LCD_W / 2) + 1, thisWidget.zone.y + 106, SOLID , CUSTOM_COLOR)     
    

    
    ------------------- PLANE COORDINATES-----------------------
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x + 240, thisWidget.zone.y + 52, "Lat:", CUSTOM_COLOR + MIDSIZE +SHADOWED)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,252,0))
    lcd.drawText(thisWidget.zone.x + 295, thisWidget.zone.y + 52, Plane_NS.." "..math.abs(Plane_Lat), CUSTOM_COLOR + MIDSIZE +SHADOWED)    

    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x + 240, thisWidget.zone.y + 77, "Lon:", CUSTOM_COLOR + MIDSIZE +SHADOWED)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,0))
    lcd.drawText(thisWidget.zone.x + 295, thisWidget.zone.y + 77, Plane_EW.." " ..math.abs(Plane_Long), CUSTOM_COLOR + MIDSIZE +SHADOWED)    
        
    
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 107, thisWidget.zone.w + 12, thisWidget.zone.y + 107, SOLID , CUSTOM_COLOR)
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 109, thisWidget.zone.w + 12, thisWidget.zone.y + 109, SOLID , CUSTOM_COLOR)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawLine(thisWidget.zone.x -4, thisWidget.zone.y + 108, thisWidget.zone.w + 13, thisWidget.zone.y + 108, SOLID , CUSTOM_COLOR)        
    
    -------------------- PILOT PARAMETERS -----------------------
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x , thisWidget.zone.y + 110, "Hdg:", CUSTOM_COLOR + MIDSIZE + SHADOWED)  
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 55 , thisWidget.zone.y + 110, math.floor(Pilot_Hdg).." degrees" , CUSTOM_COLOR + MIDSIZE + SHADOWED)  
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x , thisWidget.zone.y + 135, "Spd:", CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 55, thisWidget.zone.y + 135, math.floor(Pilot_Speed).." "..SPD, CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x , thisWidget.zone.y + 160, "Alt:", CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 55, thisWidget.zone.y + 160, math.floor(Pilot_Alt).." "..FM, CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    
   
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawLine((LCD_W / 2), thisWidget.zone.y + 109, (LCD_W / 2), thisWidget.zone.y + 188, SOLID , CUSTOM_COLOR)
    lcd.drawLine((LCD_W / 2)  + 2, thisWidget.zone.y + 109, (LCD_W / 2) + 2, thisWidget.zone.y + 188, SOLID , CUSTOM_COLOR)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawLine((LCD_W / 2) + 1, thisWidget.zone.y + 108, (LCD_W / 2) + 1, thisWidget.zone.y + 189, SOLID , CUSTOM_COLOR)  
    
    
    -------------------- PLANE PARAMETERS -----------------------
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x +240 , thisWidget.zone.y + 110, "Hdg:", CUSTOM_COLOR + MIDSIZE + SHADOWED)  
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 300 , thisWidget.zone.y + 110, math.floor(headingDeg).." degrees" , CUSTOM_COLOR + MIDSIZE + SHADOWED)  
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x +240 , thisWidget.zone.y + 135, "Spd:", CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 300, thisWidget.zone.y + 135, math.floor(Plane_Speed).." "..SPD, CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x +240 , thisWidget.zone.y + 160, "Alt:", CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 300, thisWidget.zone.y + 160, math.floor(Plane_Alt).." "..FM, CUSTOM_COLOR + MIDSIZE + SHADOWED)   
    
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 188, thisWidget.zone.w + 12, thisWidget.zone.y + 188, SOLID , CUSTOM_COLOR)
    lcd.drawLine(thisWidget.zone.x -3, thisWidget.zone.y + 190, thisWidget.zone.w + 12, thisWidget.zone.y + 190, SOLID , CUSTOM_COLOR)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(0,0,0))
    lcd.drawLine(thisWidget.zone.x -4, thisWidget.zone.y + 189, thisWidget.zone.w + 13, thisWidget.zone.y + 189, SOLID , CUSTOM_COLOR)      
    
    
    
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x + 90, thisWidget.zone.y + 187, "Distance:", CUSTOM_COLOR + DBLSIZE + SHADOWED)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 240, thisWidget.zone.y + 187, math.floor(PilotToPlaneDistance).." "..FM, CUSTOM_COLOR + DBLSIZE + SHADOWED)

    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,252,248))
    lcd.drawText(thisWidget.zone.x + 90, thisWidget.zone.y + 217, "Bearing:", CUSTOM_COLOR + DBLSIZE + SHADOWED)
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,248))
    lcd.drawText(thisWidget.zone.x + 240, thisWidget.zone.y + 217, math.floor(Bearing).." deg", CUSTOM_COLOR + DBLSIZE + SHADOWED)
    
    
    ------------------------------------------- DEBUG --------------------------------------------------
    --lcd.drawNumber(thisWidget.zone.x, thisWidget.zone.y, Pilot_GPSTable.numsat, LEFT + TEXT_COLOR + SHADOWED);
  
    --lcd.drawText(thisWidget.zone.x + 300, thisWidget.zone.y + 230, thisWidget.zone.w.."x"..thisWidget.zone.h, TEXT_COLOR + SHADOWED);


  else
  
    lcd.setColor(CUSTOM_COLOR, lcd.RGB(248,0,0))
    lcd.drawText(thisWidget.zone.x , thisWidget.zone.y + (thisWidget.zone.h / 2), "FULL SCREEN ONLY", CUSTOM_COLOR + SMLSIZE + SHADOWED)
  
  end
  --  if Plane_Sats > 0  then
  --    lcd.drawText(thisWidget.zone.x, thisWidget.zone.x + 175, "Satellites: "..math.floor(Plane_Sats), CUSTOM_COLOR + MIDSIZE + SHADOWED)  
  --  end
  
end

return { name="X12SGPS", options=options, create=create, update=update, background=background, refresh=refresh }